//
//  STColor.h
//  iCoda
//
//  Created by Davyd Geyl on 30/08/08.
//  Copyright 2008 Sportstec Ltd. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@interface STColor : NSObject <NSCoding>
{
	NSColor* _color;
	CGColorRef _CGColor;
}

typedef struct RGBAComponents
{
	CGFloat red;
	CGFloat green;
	CGFloat blue;
	CGFloat alpha;
} RGBAComponents;

+ (instancetype)blackColor;
+ (instancetype)darkGrayColor;	/* 0.333 white */
+ (instancetype)lightGrayColor;	/* 0.667 white */
+ (instancetype)whiteColor;	/* 1.0 white */
+ (instancetype)grayColor;		/* 0.5 white */
+ (instancetype)redColor;		/* 1.0, 0.0, 0.0 RGB */
+ (instancetype)greenColor;	/* 0.0, 1.0, 0.0 RGB */
+ (instancetype)blueColor;		/* 0.0, 0.0, 1.0 RGB */
+ (instancetype)cyanColor;		/* 0.0, 1.0, 1.0 RGB */
+ (instancetype)yellowColor;	/* 1.0, 1.0, 0.0 RGB */
+ (instancetype)magentaColor;	/* 1.0, 0.0, 1.0 RGB */
+ (instancetype)orangeColor;	/* 1.0, 0.5, 0.0 RGB */
+ (instancetype)purpleColor;	/* 0.5, 0.0, 0.5 RGB */
+ (instancetype)brownColor;	/* 0.6, 0.4, 0.2 RGB */
+ (instancetype)clearColor;	/* 0.0 white, 0.0 alpha */

+ (instancetype)colorWithCGColor:(CGColorRef)colorRef;
+ (instancetype)colorWithColorObject:(id)colorObject;
+ (instancetype)colorWithRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue alpha:(CGFloat)alpha;
+ (instancetype)colorWithRGBAComponents:(RGBAComponents)RGBAComponents;
+ (instancetype)colorWithHue:(CGFloat)hue saturation:(CGFloat)saturation brightness:(CGFloat)brightness alpha:(CGFloat)alpha;
+ (instancetype)colorWithAnotherColor:(STColor*)anotherColor;
+ (instancetype)colorWithAnotherColorBlendingWithWhite:(STColor*)anotherColor;
+ (instancetype)colorWithWhite:(CGFloat)white alpha:(CGFloat)alpha;

- (instancetype)initWithColorObject:(id)colorObject;
- (instancetype)initWithRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue alpha:(CGFloat)alpha;
- (instancetype)initWithHue:(CGFloat)hue saturation:(CGFloat)saturation brightness:(CGFloat)brightness alpha:(CGFloat)alpha;
- (instancetype)initWithAnotherColor:(STColor*)anotherColor;
- (instancetype)initWithJSONRepresentation:(NSString *)JSONRepresentation;

@property (nonatomic, readonly) NSString *JSONStringRepresentation;

// Parameters must not be nil
- (void)getRed:(CGFloat*)red green:(CGFloat*)green blue:(CGFloat*)blue alpha:(CGFloat*)alpha;

-(void)getHue:(CGFloat*)hue saturation:(CGFloat*)saturation brightness:(CGFloat*)brightness alpha:(CGFloat*)alpha;

- (id)colorObject;
- (CGColorRef)CGColor;

- (void)set;
- (void)setFill;
- (void)setStroke;

- (STColor*)blendedColorWithFraction:(CGFloat)fraction ofColor:(STColor*)color;

- (CGFloat)hueEquivalentValue;
- (CGFloat)saturationEquivalentValue;
- (CGFloat)brightnessEquivalentValue;
- (CGFloat)alphaEquivalentValue;

- (CGFloat)alphaComponent;

- (STColor*)simpleGradientEndColor;

// Comparison
- (NSComparisonResult)compareByComponents:(STColor*)color;

- (BOOL)isEqualToColor:(STColor*)color;

@end
