//
//  STSorting.h
//  Kahuna
//
//  Created by Davyd Geyl on 4/04/11.
//  Copyright 2011 Sportstec Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>

@protocol STSortMasterInterface;

@protocol STSortableItemInterface <NSObject>

- (NSUInteger)orderIndex;
- (void)setOrderIndex:(NSUInteger)index;

- (id<STSortMasterInterface>)sortMaster;

@end

extern NSString* const STSortMasterDidUpdateSortingNotification;

@protocol STSortMasterInterface <NSObject>
@optional

- (void)placeUserItem:(id<STSortableItemInterface>)item afterItem:(id<STSortableItemInterface>)precedingItem;
- (void)placeUserItems:(NSArray*)items afterItem:(id<STSortableItemInterface>)precedingItem;

- (void)decreaseOrderIndexOfItem:(id<STSortableItemInterface>)item;
- (void)moveItemToLeastOrderIndex:(id<STSortableItemInterface>)item;
- (void)increaseOrderIndexOfItem:(id<STSortableItemInterface>)item;
- (void)moveItemToGreatestOrderIndex:(id<STSortableItemInterface>)item;

- (BOOL)isItemMovableToGreatestOrderIndex:(id<STSortableItemInterface>)item;
- (BOOL)isItemMovableToLeastOrderIndex:(id<STSortableItemInterface>)item;

- (NSUInteger)nextOrderIndex;

- (NSSortDescriptor*)customSortDescriptor;

- (void)updateSorting;

@end

@interface NSObject (STSortMasterInterface)

// Required methods
- (NSMutableArray*)sortedItems;

// Optional methods
- (NSUndoManager*)undoManager;
- (NSString*)customSortDescriptorKey; // "orderIndex" key used by default

- (void)placeUserItem:(id<STSortableItemInterface>)item afterItem:(id<STSortableItemInterface>)precedingItem;

- (void)placeUserItem:(id<STSortableItemInterface>)item beforeItem:(id<STSortableItemInterface>)afterItem;

@end
